<?php

/**
 * Helper to get the location post type(s) from theme-supports config and normalize into an array.
 *
 * @return array
 */
function bt_locator_post_types() {
	$location_post_type = bt_use_locator( 'location_post_type' );

	// must be string or array
	if ( empty( $location_post_type )
		|| ( ! is_array( $location_post_type ) && ! is_string( $location_post_type ) )
	) {
		return [];
	}

	if ( is_string( $location_post_type ) ) {
		// put in an array
		return [$location_post_type];
	}

	// already an array
	return $location_post_type;
}


/**
 * Determine the post type name for dynamic controller matching.
 *
 * @param \WP_Post $post
 *
 * @return string
 */
function bt_location_post_type_controller_name( $post ) {
	// underscores/hyphens to spaces => uppercase words => remove spaces
	return str_replace( ' ', '', ucwords( str_replace( ['_','-'], ' ', $post->post_type ) ) );
}


/**
 * Fetch and return numeric zip from request params if exists. Used by front end for search form.
 *
 * @return string
 */
function bt_request_zip_code() {
	$zip = '';

	if ( isset( $_REQUEST['zip'] ) && is_numeric( $_REQUEST['zip'] ) ) {
		$zip = $_REQUEST['zip'];
	}

	return $zip;
}


/**
 * Fetch and return numeric radius from request params if exists. Used by front end for search form.
 *
 * @return string
 */
function bt_request_radius() {
	$radius = '10';

	if ( isset( $_REQUEST['radius'] ) && is_numeric( $_REQUEST['radius'] ) ) {
		$radius = $_REQUEST['radius'];
	}

	return $radius;
}


/**
 * Look up city (locality) for a lat lng. Ported in from VenYou. Unused as of initial plugin build, but kept here in
 * case it's needed in the future.
 *
 * @param $strLatLng string The two coordinates concatenated by a comma.
 *
 * @return mixed String for city/locality name if success, false if failure.
 */
function bt_lat_lng_city($strLatLng) {
	$strLatLngParams = 'latlng=' . $strLatLng . '&sensor=false';
	$objLatLngResult = bt_google_maps_geocode( $strLatLngParams );
	if ($objLatLngResult) {
		$objLatLngResult = $objLatLngResult->results[0]->address_components;
		foreach ($objLatLngResult as $objAddressComponent) {
			if ($objAddressComponent->types[0] == 'locality') {
				// found the locality/city item; return the name of it
				return $objAddressComponent->long_name;
			}
		}
		// didn't find locality for some reason; return lat/lng as fallback
		return $strLatLng;
	} else {
		return false;
	}
}


/**
 * Look up lat lng for a zip. Ported in from VenYou.
 *
 * @param $strZip string A 5-digit zip code, though it will get padded with zeroes if less than 5.
 *
 * @return array|bool If success, an array with two values: lat and lng. False on failure.
 */
function bt_zip_lat_lng($strZip) {
	$strLatLngParams = 'address=' . urlencode(str_pad($strZip, 5, '0', STR_PAD_LEFT) . ', USA') . '&sensor=false';
	$objLatLngResult = bt_google_maps_geocode( $strLatLngParams );
	if ($objLatLngResult) {
		$objLatLngResult = $objLatLngResult->results[0]->geometry->location;
		return array($objLatLngResult->lat, $objLatLngResult->lng);
	} else {
		return false;
	}
}


/**
 * Look up lat lng for an address. Unused as of initial plugin build, but kept here in case it's needed in the future.
 *
 * @param $address string An address like "Grand Rapids, MN" (", USA" will be appended).
 *
 * @return array|bool If success, an array with two values: lat and lng. False on failure.
 */
function address_lat_lng( $address ) {
	$strLatLngParams = 'address=' . urlencode($address . ', USA') . '&sensor=false';
	$objLatLngResult = bt_google_maps_geocode( $strLatLngParams );
	if ($objLatLngResult) {
		$objLatLngResult = $objLatLngResult->results[0]->geometry->location;
		return array($objLatLngResult->lat, $objLatLngResult->lng);
	} else {
		error_log(__FUNCTION__.' -- lat/lng fetch failed! Result: '.print_r($objLatLngResult,1));
		return false;
	}
}


/**
 * Make a request to Google Maps Geocode API and return the result as an object
 *
 * @param $params string URL query string of parameters for the maps request
 *
 * @return array|mixed|object On success, an stdClass object of the request response
 */
function bt_google_maps_geocode( $params ) {
	// initialize curl request to google maps
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, 'https://maps.googleapis.com/maps/api/geocode/json?' . $params . '&key='.GOOGLE_MAPS_API_KEY);
	curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);		// return into a variable
	curl_setopt($ch, CURLOPT_TIMEOUT, 3);				// times out after X secs
	curl_setopt($ch, CURLOPT_FORBID_REUSE, true);		//forces closure of connection when done
	// attempt curl request (multiple times if no response)
	$i=1;
	while ($i++ <= 3) {
		$strLatLngResult = curl_exec($ch);
		$arrLatLngReturnHeaders = curl_getinfo($ch);
		if ($arrLatLngReturnHeaders['http_code'] != 200) {
			// wait X seconds to see if its a temporary network issue.
			sleep(2);
		} else if ($arrLatLngReturnHeaders['http_code'] == 200) {
			// we got a good response, drop out of loop.
			break;
		}
	}
	// turn result into object
	$objLatLngResult = json_decode($strLatLngResult);
	if ($objLatLngResult->status == "OK") {
		return $objLatLngResult;
	} else {
		$strDebugMsg = "Result = " . print_r($objLatLngResult,true) . "\r\n\r\n";
		$strDebugMsg .= "Query = " . $params . "\r\n";
		bt_email_alert( 'Google Maps API Lookup Error', $strDebugMsg );
		return false;
	}
}


/**
 * Build the URL for a map & directions link
 *
 * @param array $address The data from ACF address field.
 *
 * @return string HTML for output.
 */
function bt_directions_url( $address ) {
	return 'https://www.google.com/maps/search/' . urlencode( $address['address'] );
}


/**
 * Get a standardized address object for a post
 *
 * @param $post int|\WP_Post A numeric ID for a post or WP_Post object
 *
 * @return \stdClass The address object which should have members address (City, ST), lat, lng
 */
function bt_address_object_for_post( $post ) {
	$address = get_field( 'street_address', $post );
	if ( is_array( $address ) ) {
		// sometimes the data is an array so we convert to an object if so.
		$address = (object)$address;
	}
	return $address;
}


/* ---------- things that should go in a bt-helpers plugin ---------- */

/**
 * Helper function to send an internal email alert
 *
 * @param $subject string The subject of the message.
 * @param $message string Plain-text body of the message to be sent.
 */
if ( ! function_exists( 'bt_email_alert' ) ) {
	function bt_email_alert( $subject, $message ) {
		$to = [
			'Jeremy Burgeson <jeremy@bicycletheory.com>',
		];
		$headers = [
			'From: ' . get_bloginfo( 'name' ) . ' Website <no-reply@bicycletheory.com>',
		];

		wp_mail( $to, $subject, $message, $headers );
	}
}
